/**********************************************************************************************/
/* OBL_SQL.h																	 	  		  */
/*                                                                       					  */
/* Object Base Library ( OBL )			                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                        					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __OBL_SQL_h
   #define __OBL_SQL_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/OBL.h>
#	include <VSDK/FBL_String.h>
#else
#	include <OBL.h>
#	include <FBL_String.h>
#endif // VSDK_FRAMEWORK_STYLE

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
enum ECursorLocation 
{
	kClient = 1,	// Client side cursor
	kServer,		// Server side cursor
	kServerBulk		// Server side read-only cursor that can load records in batch mode.
};


/**********************************************************************************************/
enum ECursorDirection
{
	kForwardOnly = 1,	// navigation can be only from first to last records. ONCE.
	kRandom				// navigation can be in both directions. many times.
};


/**********************************************************************************************/
class FBL_BaseField;
class OBL_Object;


/**********************************************************************************************/
enum OBL_ColumnOperation
{
	opUndefined = 0,
	opLink,
	opAnd, opOr, opNot,
	opEquals, opLess, opGreater, opLessEqual, opGreaterEqual, opNotEqual, 
	opLike, opRegEx,
	opNull, opNotNull,
	opFuncLeft,
	opLeftOuterJoin, opRightOuterJoin, opFullOuterJoin,

	// this imaginary operations are not mirrowed in the opInSubselect
	opInSubselect
};

enum OBL_SQLtoken
{
	// First elements of this enum MUST be as in the enum OBL_ColumnOperation
	tokUndefined	= 0, 
	tokLink,
	tokAnd, tokOr, tokNot,
	tokEquals, tokLess, tokGreater, tokLessEqual, tokGreaterEqual, tokNotEqual,
	tokLike, tokRegEx,
	tokNull, tokNotNull,
	tokFuncLeft,
	tokLeftOuterJoin, tokRightOuterJoin, tokFullOuterJoin,

	// now other tokens
	tokEnd,
	tokComma,

	tokIn, tokBeetwen,

	tokShiftLeft, tokShiftRight,
	tokPlus, tokMinus, 
	tokMul, tokDiv, tokMod,
	
	tokLeftBracket,	tokRightBracket,
	tokLeftSquerBracket, tokRightSquerBracket,

	tokSelect, tokFrom, tokWhere, tokOrderBy, tokGroupBy, tokHaving, 
	tokIs,
	tokAsc,	tokDesc,
	tokAsteric, tokAstericAsteric,

	tokField, tokAllFields, tokAllFieldsMethods,
	tokAs,
	tokBaseObject,

	tokValue,
	tokName,
	tokNumericConstant, tokDoubleConstant, tokStringConstant,

	tokNoCase,
	tokDistinct, tokAll,

	tokCountAll, tokCount, tokSum, tokAvg, 
	tokMin, tokMax	
};


/**********************************************************************************************/
struct OBL_FieldConstraints
{
	OBL_FieldConstraints *left, 
						 *right,
						 *parent;
	
	
	bool				mNot;				// true if after resolve of branch we must 
											// invert selection.
	
	OBL_ColumnOperation	columnOperation;
	OBL_Object*			mObject;			// BaseObject or its psevdonim.
 
	FBL_BaseField*		field;		
	FBL_String			value;
	ulong 				param;	// additional paramter for value, for example "match case"

				OBL_FieldConstraints( OBL_ColumnOperation inType = opUndefined ) {
					left			= NULL;
					right			= NULL;
					parent			= NULL;
					mNot			= false;
					columnOperation = inType;
					mObject			= NULL;
					field			= NULL;
					param			= 0;
				}

				~OBL_FieldConstraints( void ) {
					ForgetObject( left );
					ForgetObject( right );
				}			
};


/**********************************************************************************************/
const long errSQL_Expected_SELECT		= 600;
const long errSQL_Expected_FROM			= 601;

const long errSQL_TableNotFound			= 602;
const long errSQL_FieldNotFound			= 603;

const long errSQL_SyntaxError			= 604;
const long errSQL_ReadOnly				= 605;
const long errSQL_Expected_NULL			= 606;
const long errSQL_Expected_LeftBracket	= 607;
const long errSQL_Expected_RightBracket	= 608;
const long errSQL_Expected_Expression	= 609;
const long errSQL_WrongFieldType		= 610;
const long errSQL_NotStringField		= 611;
const long errSQL_Expected_Value		= 612;

const long errSQL_WrongLink				= 613;
const long errSQL_AmbiguousLink			= 614;
const long errSQL_MissingLink			= 615;
const long errSQL_WrongExpression		= 616;

const long errSQL_MixAgregativeAndNormal= 617;
const long errSQL_Expected_Comma 		= 618;
const long errSQL_NotEnoughParameters	= 619;

const long errSQL_Expected_TABLE		= 620;
const long errSQL_Expected_WHERE		= 621;
const long errSQL_Expected_INTO			= 622;
const long errSQL_Expected_VALUES		= 623;
const long errSQL_Expected_SET			= 624;
const long errSQL_Expected_ASSIGN		= 625;
const long errSQL_Expected_DOT			= 626;

const long errSQL_Invalid_FieldLength	= 627;
const long errSQL_Invalid_FieldValue	= 628;
const long errSQL_Missing_FieldValues	= 629;
const long errSQL_ExpectedStringConstant = 630;
const long errSQL_Expected_AND 			= 631;
const long errSQL_Expected_ONE_FIELD 	= 632;


// semantic errors:
const long errSQL_FieldNotFromSelectedTable	= 640;


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif
