/**********************************************************************************************/
/* VDK_Cursor.h																				  */
/*                                                                       					  */
/* Valentina SDK 1.3																		  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __VDK_Cursor_h
   #define __VDK_Cursor_h

#if VSDK_FRAMEWORK_STYLE
#include <VSDK/FBL.h> 
#include <VSDK/FBL_String.h> 
#include <VSDK/OBL_SQL.h> 
#include <VSDK/VDK_Boolean.h>
#include <VSDK/VDK_Byte.h>
#include <VSDK/VDK_Short.h>
#include <VSDK/VDK_UShort.h>
#include <VSDK/VDK_Medium.h>
#include <VSDK/VDK_UMedium.h>
#include <VSDK/VDK_Long.h>
#include <VSDK/VDK_ULong.h>
#include <VSDK/VDK_LLong.h>
#include <VSDK/VDK_ULLong.h>
#include <VSDK/VDK_Float.h>
#include <VSDK/VDK_Double.h>
#include <VSDK/VDK_Date.h>
#include <VSDK/VDK_Time.h>
#include <VSDK/VDK_DateTime.h>
#include <VSDK/VDK_String.h>
#include <VSDK/VDK_VarChar.h>
#include <VSDK/VDK_FixedBinary.h>
#include <VSDK/VDK_VarBinary.h>
#include <VSDK/VDK_BLOB.h>
#include <VSDK/VDK_Text.h>
#include <VSDK/VDK_Picture.h>
#else
#include <FBL.h> 
#include <FBL_String.h> 
#include <OBL_SQL.h> 
#include <VDK_Boolean.h>
#include <VDK_Byte.h>
#include <VDK_Short.h>
#include <VDK_UShort.h>
#include <VDK_Medium.h>
#include <VDK_UMedium.h>
#include <VDK_Long.h>
#include <VDK_ULong.h>
#include <VDK_LLong.h>
#include <VDK_ULLong.h>
#include <VDK_Float.h>
#include <VDK_Double.h>
#include <VDK_Date.h>
#include <VDK_Time.h>
#include <VDK_DateTime.h>
#include <VDK_String.h>
#include <VDK_VarChar.h>
#include <VDK_FixedBinary.h>
#include <VDK_VarBinary.h>
#include <VDK_BLOB.h>
#include <VDK_Text.h>
#include <VDK_Picture.h>
#endif // VSDK_FRAMEWORK_STYLE


#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class OBL_Cursor;

class VDK_BaseObject;
class VDK_DataBase;


/**********************************************************************************************/
typedef FBL_Array<VDK_Field*> 			FieldsArray;


/**********************************************************************************************/
class VDK_Cursor
{
	public://///////////////////////////////////////////////////////////////////////////////////

		/* Construction/Destruction: */

FBL_EXP							VDK_Cursor( 
									VDK_DataBase* 		inDataBase,
									ECursorLocation		inLocation	= kClient, 
									ELockType 			inType		= kReadOnly,
									ECursorDirection 	inDirection = kForwardOnly );

FBL_EXP virtual					~VDK_Cursor( void );
                            
virtual	VDK_Cursor*				Clone( void ) { return NULL; }

								// Returns true if at least one existing VDK_Database has a pointer
								// on a specified inCursor. Can be used for debugging.
FBL_EXP static 	bool			Exists( VDK_Cursor* inCursor );

		/* Accesories: */
		
		VDK_DataBase*			GetDataBase( void ) const { return mDataBase; }
FBL_EXP	FBL_String				GetSQLString( void ) const;
		
FBL_EXP	bool					IsReadOnly( void ) const;
		OBL_Cursor*				GetImplementation( void ) const { return mImplementation; }


		/* SQL */
								// Parse SQL string and execute SQL query
FBL_EXP	long					ExecuteSQL( const char* inSQLstring );

								// Parse SQL string but don't execute it.
FBL_EXP	long					PrepareSQL( const char* inSQLstring );
		
								// Execute parsed SQL query as many times as it needs. 
FBL_EXP	long					ExecuteSQL( void );	


		/* Navigation methods: */

FBL_EXP	bool					FirstRecord( void );
FBL_EXP	bool					LastRecord( void );

FBL_EXP	bool					NextRecord( void );
FBL_EXP	bool					PrevRecord( void );

FBL_EXP	bool					GoToRecord( ulong inRecIndex );
FBL_EXP	ulong					GetPosition( void ) const;

		bool					GetEOF( void ) const { return mWidePos > GetRecordCount(); }
		bool					GetBOF( void ) const { return mWidePos == 0; }	


		/* Record methods: */

								// Returns number of selected records 
FBL_EXP	ulong					GetRecordCount( void ) const;
		
FBL_EXP	bool					Add( void );

FBL_EXP	bool					Delete( void );
FBL_EXP	bool					DeleteAll( void );		

FBL_EXP	void					DropRecord( void );

FBL_EXP	bool					Update( void );

								// Do VERY FAST update of all cursor records using values of
								// one or several fields which just only was assigned (dirty fields).
								// This function is analog of SQL commanad: 
								// 		"UPDATE ... SET ... WHERE ..."
FBL_EXP	bool 					UpdateAll( void );

		
								// Reduce number of records in the Cursor
FBL_EXP	void					ReduceSelection( ulong inRecords );
		
FBL_EXP	void					SetBlank( void );	
	
								/** This method allow change Read lock of cursor record
									to Write lock so we can edit THIS ONE record. 
									Returns FALSE, iif this cannot be done for the record. */
FBL_EXP	bool					SetWriteLock( ulong inRecordIndex );	
			

		/* Field methods: */
		
		ushort					GetFieldCount( void ) const{ return (ushort) mFields.GetCount();	}
		
		VDK_Field*				GetField( ushort inIndex ) const;
FBL_EXP	VDK_Field*				GetField( const char* inName ) const;
		

		// Methods that try to cast type of field to expected TYPE*.
		// If fail return NULL.

		bool					GetBoolean	( ushort 		inIndex ) const;
		bool					GetBoolean	( const char* 	inName 	) const;

		uchar					GetByte		( ushort 		inIndex ) const;
		uchar					GetByte		( const char* 	inName 	) const;

		short					GetShort	( ushort 		inIndex ) const;
		short					GetShort	( const char* 	inName 	) const;

		ushort					GetUShort	( ushort 		inIndex ) const;
		ushort					GetUShort	( const char* 	inName 	) const;

		long					GetMedium	( ushort 		nIndex ) const;
		long					GetMedium	( const char* 	inName ) const;

		ulong					GetUMedium	( ushort 		inIndex ) const;
		ulong					GetUMedium	( const char* 	inName 	) const;

		long					GetLong		( ushort 		nIndex ) const;
		long					GetLong		( const char* 	inName ) const;

		ulong					GetULong	( ushort 		inIndex ) const;
		ulong					GetULong	( const char* 	inName 	) const;

		llong					GetLLong	( ushort 		nIndex ) const;
		llong					GetLLong	( const char* 	inName ) const;

		ullong					GetULLong	( ushort 		inIndex ) const;
		ullong					GetULLong	( const char* 	inName 	) const;

		float					GetFloat	( ushort 		inIndex ) const;
		float					GetFloat	( const char* 	inName 	) const;

		double					GetDouble	( ushort 		inIndex ) const;
		double					GetDouble	( const char* 	inName 	) const;

		void					GetDate		( ushort 		inIndex, 
											  	long& outYear, ushort& outMonth, ushort& outDay ) const;
		void					GetDate		( const char* 	inName, 
												long& outYear, ushort& outMonth, ushort& outDay ) const;

		void					GetTime		( ushort 		inIndex, 
												ushort& outHour, ushort& outMinute, ushort& outSecond ) const;
		void					GetTime		( const char* 	inName, 
												ushort& outHour, ushort& outMinute, ushort& outSecond ) const;

								// GetString works for any type of field, which can be represented as string.
								// You must delete returned FBL_String. 
								// If value of a field is NULL then returns NULL pointer.  
FBL_EXP	FBL_String*				GetString	( ushort 		inIndex ) const;
FBL_EXP	FBL_String*				GetString	( const char* 	inName 	) const;

		PicHandle				GetPicture	( ushort 		inIndex ) const;
		PicHandle				GetPicture	( const char* 	inName 	) const;


			// Methods which try to cast type of field to expected one.

		VDK_Boolean*			GetBooleanField( ushort inIndex ) const;
		VDK_Boolean*			GetBooleanField( const char* inName ) const;

		VDK_Byte*				GetByteField( ushort inIndex ) const;
		VDK_Byte*				GetByteField( const char* inName ) const;

		VDK_Short*				GetShortField( ushort inIndex ) const;
		VDK_Short*				GetShortField( const char* inName ) const;

		VDK_UShort*				GetUShortField( ushort inIndex ) const;
		VDK_UShort*				GetUShortField( const char* inName ) const;

		VDK_Medium*				GetMediumField( ushort inIndex ) const;
		VDK_Medium*				GetMediumField( const char* inName ) const;

		VDK_UMedium*			GetUMediumField( ushort inIndex ) const;
		VDK_UMedium*			GetUMediumField( const char* inName ) const;

		VDK_Long*				GetLongField( ushort inIndex ) const;
		VDK_Long*				GetLongField( const char* inName ) const;

		VDK_ULong*				GetULongField( ushort inIndex ) const;
		VDK_ULong*				GetULongField( const char* inName ) const;

		VDK_LLong*				GetLLongField( ushort inIndex ) const;
		VDK_LLong*				GetLLongField( const char* inName ) const;

		VDK_ULLong*				GetULLongField( ushort inIndex ) const;
		VDK_ULLong*				GetULLongField( const char* inName ) const;

		VDK_Float*				GetFloatField( ushort inIndex ) const;
		VDK_Float*				GetFloatField( const char* inName ) const;

		VDK_Double*				GetDoubleField( ushort inIndex ) const;
		VDK_Double*				GetDoubleField( const char* inName ) const;

		VDK_Date*				GetDateField( ushort inIndex ) const;
		VDK_Date*				GetDateField( const char* inName ) const;

		VDK_Time*				GetTimeField( ushort inIndex ) const;
		VDK_Time*				GetTimeField( const char* inName ) const;

		VDK_DateTime*			GetDateTimeField( ushort inIndex ) const;
		VDK_DateTime*			GetDateTimeField( const char* inName ) const;

		VDK_String*				GetStringField( ushort inIndex ) const;
		VDK_String*				GetStringField( const char* inName ) const;

		VDK_VarChar*			GetVarCharField( ushort inIndex ) const;
		VDK_VarChar*			GetVarCharField( const char* inName ) const;

		VDK_FixedBinary*		GetFixedBinaryField( ushort inIndex ) const;
		VDK_FixedBinary*		GetFixedBinaryField( const char* inName ) const;

		VDK_VarBinary*			GetVarBinaryField( ushort inIndex ) const;
		VDK_VarBinary*			GetVarBinaryField( const char* inName ) const;

		VDK_BLOB*				GetBLOBField( ushort inIndex ) const;
		VDK_BLOB*				GetBLOBField( const char* inName ) const;

		VDK_Text*				GetTextField( ushort inIndex ) const;
		VDK_Text*				GetTextField( const char* inName ) const;

		VDK_Picture*			GetPictureField( ushort inIndex ) const;
		VDK_Picture*			GetPictureField( const char* inName ) const;


		/* Import/Export: */
		
FBL_EXP	void					ImportASCII( 
									const char*		inFile, 
									char			inFldDelimeter 	= '\t',
									char			inLineDelimeter = 13,
									const char*		inTaskName 		= NULL );

FBL_EXP	void					ExportASCII( 
									const char*		inFile, 
									char			inFldDelimeter 	= '\t',
									char			inLineDelimeter	= 13,
									const char*		inTaskName 		= NULL );

#ifdef FBL_MAC
FBL_EXP	void					ImportASCII( 
									const FSSpec*	inFile, 
									char			inFldDelimeter 	= '\t',
									char			inLineDelimeter = 13,
									const char*		inTaskName 		= NULL );

FBL_EXP	void					ExportASCII( 
									const FSSpec*	inFile, 
									char			inFldDelimeter 	= '\t',
									char			inLineDelimeter	= 13,
									const char*		inTaskName 		= NULL );
#endif


		/* Row buffer methods: */
		
/*		FBL_CursorDesc* 		GetCursorDescription( void );
		void* 					GetRow( ulong& outBufferSize );
*/

	protected://////////////////////////////////////////////////////////////////////////////////

FBL_EXP	void					Find_VDK_Objects( void );


	private:////////////////////////////////////////////////////////////////////////////////////
 
		const char*				FindNextUnion( const char* inSQLstring );
		
		long 					ExecuteSql_WithUnion( 
									const char* inSQLstring, 
									const char* inUnionPtr );
									
		long 					ExecuteSql_WithNoUnion( 
									const char* inSQLstring );
									
		OBL_Cursor*				ExecuteSql_Single( 
									const char* inSQLstring ) ;


	protected://////////////////////////////////////////////////////////////////////////////////

		/* Description of cursor */
		
		VDK_DataBase*			mDataBase;
		
		OBL_Cursor*				mImplementation;

		FieldsArray				mFields;		// Fields from SELECT statement.


	private:////////////////////////////////////////////////////////////////////////////////////

		VDK_BaseObject*			mTmpBaseObject; 
		ulong					mWidePos; 		// pos counter for EOF and BOF support.
					 		

	public://///////////////////////////////////////////////////////////////////////////////////
 
		
		// These 2 data members allows easy to use VSDK to develop plugins
		// such as Valentina for REALbasic, Valentina for Java, which do have garbage collection.
			
			// An owner should set up this back reference, so when destructor of this class
			// will be called FROM kernel it will set back ref to NULL via mBackRef. 
			// The owner will be able to handle correctly this situation and won't try access
			// already destroyed kernel object.
		VDK_Cursor**			mBackRef;
				
			// here you can put pointer on your own additional data (or even structure)
			// if you need attach several variables to this C++ instance.	
		void*					mXtraData;	
		
			// Destructor ~VDK_Cursor() will call this pointer if you set it.
		void					(*mpFinalize)(VDK_Cursor* inCursor );
};


/**********************************************************************************************/
inline VDK_Field* VDK_Cursor::GetField( ushort inIndex ) const 
{ 
	if( 0 < inIndex && inIndex <= mFields.GetCount() )
			return mFields.GetItemAt(inIndex);	
	else 	return NULL;	
}


#pragma mark -
 

/**********************************************************************************************/
inline bool VDK_Cursor::GetBoolean( ushort inIndex ) const{ 
	VDK_Boolean* pBoolean = GetBooleanField( inIndex );
	return (pBoolean) ? bool(*pBoolean) : false;
}

/**********************************************************************************************/
inline bool VDK_Cursor::GetBoolean( const char* inName ) const{
	VDK_Boolean* pBoolean = GetBooleanField( inName );
	return (pBoolean) ? bool(*pBoolean) : false;	
}

/**********************************************************************************************/
inline uchar VDK_Cursor::GetByte( ushort inIndex ) const{ 
	VDK_Byte* pByte = GetByteField( inIndex );
	return (pByte) ? uchar(*pByte) : uchar(0);	
}

/**********************************************************************************************/
inline uchar VDK_Cursor::GetByte( const char* inName ) const{
	VDK_Byte* pByte = GetByteField( inName );
	return (pByte) ? uchar(*pByte) : uchar(0);	
}

/**********************************************************************************************/
inline short VDK_Cursor::GetShort( ushort inIndex ) const{ 
	VDK_Short* pShort = GetShortField( inIndex );
	return (pShort) ? short(*pShort) : short(0);	
}

/**********************************************************************************************/
inline short VDK_Cursor::GetShort( const char* inName ) const{
	VDK_Short* pShort = GetShortField( inName );
	return (pShort) ? short(*pShort) : short(0);	
}

/**********************************************************************************************/
inline ushort VDK_Cursor::GetUShort( ushort inIndex ) const{ 
	VDK_UShort* pUShort = GetUShortField( inIndex );
	return (pUShort) ? ushort(*pUShort) : ushort(0);	
}

/**********************************************************************************************/
inline ushort VDK_Cursor::GetUShort( const char* inName ) const{
	VDK_UShort* pUShort = GetUShortField( inName );
	return (pUShort) ? ushort(*pUShort) : ushort(0);	
}

/**********************************************************************************************/
inline long VDK_Cursor::GetMedium( ushort inIndex ) const{ 
	VDK_Medium* pMedium = GetMediumField( inIndex );
	return (pMedium) ? long(*pMedium) : long(0);	
}

/**********************************************************************************************/
inline long VDK_Cursor::GetMedium( const char* inName ) const{
	VDK_Medium* pMedium = GetMediumField( inName );
	return (pMedium) ? long(*pMedium) : long(0);	
}

/**********************************************************************************************/
inline ulong VDK_Cursor::GetUMedium( ushort inIndex ) const{ 
	VDK_UMedium* pUMedium = GetUMediumField( inIndex );
	return (pUMedium) ? ulong(*pUMedium) : ulong(0);
}

/**********************************************************************************************/
inline ulong VDK_Cursor::GetUMedium( const char* inName ) const{
	VDK_UMedium* pUMedium = GetUMediumField( inName );
	return (pUMedium) ? ulong(*pUMedium) : ulong(0);
}

/**********************************************************************************************/
inline long VDK_Cursor::GetLong( ushort inIndex ) const{ 
	VDK_Long* pLong = GetLongField( inIndex );
	return (pLong) ? long(*pLong) : long(0);	
}

/**********************************************************************************************/
inline long VDK_Cursor::GetLong( const char* inName ) const{
	VDK_Long* pLong = GetLongField( inName );
	return (pLong) ? long(*pLong) : long(0);	
}

/**********************************************************************************************/
inline ulong VDK_Cursor::GetULong( ushort inIndex ) const{ 
	VDK_ULong* pULong = GetULongField( inIndex );
	return (pULong) ? ulong(*pULong) : ulong(0);	
}

/**********************************************************************************************/
inline ulong VDK_Cursor::GetULong( const char* inName ) const{
	VDK_ULong* pULong = GetULongField( inName );
	return (pULong) ? ulong(*pULong) : ulong(0);	
}

/**********************************************************************************************/
inline llong VDK_Cursor::GetLLong( ushort inIndex ) const{ 
	VDK_LLong* pLLong = GetLLongField( inIndex );
	return (pLLong) ? llong(*pLLong) : llong(0);	
}

/**********************************************************************************************/
inline llong VDK_Cursor::GetLLong( const char* inName ) const{
	VDK_LLong* pLLong = GetLLongField( inName );
	return (pLLong) ? llong(*pLLong) : llong(0);	
}

/**********************************************************************************************/
inline ullong VDK_Cursor::GetULLong( ushort inIndex ) const{ 
	VDK_ULLong* pULLong = GetULLongField( inIndex );
	return (pULLong) ? ullong(*pULLong) : ullong(0);	
}

/**********************************************************************************************/
inline ullong VDK_Cursor::GetULLong( const char* inName ) const{
	VDK_ULLong* pULLong = GetULLongField( inName );
	return (pULLong) ? ullong(*pULLong) : ullong(0);	
}

/**********************************************************************************************/
inline float VDK_Cursor::GetFloat( ushort inIndex ) const{ 
	VDK_Float* pFloat = GetFloatField( inIndex );
	return (pFloat) ? float(*pFloat) : float(0);	
}

/**********************************************************************************************/
inline float VDK_Cursor::GetFloat( const char* inName ) const{
	VDK_Float* pFloat = GetFloatField( inName );
	return (pFloat) ? float(*pFloat) : float(0);	
}

/**********************************************************************************************/
inline double VDK_Cursor::GetDouble( ushort inIndex ) const{ 
	VDK_Double* pDouble = GetDoubleField( inIndex );
	return pDouble ? double(*pDouble) : double(0);	
}

/**********************************************************************************************/
inline double VDK_Cursor::GetDouble( const char* inName ) const{
	VDK_Double* pDouble = GetDoubleField( inName );
	return pDouble ? double(*pDouble) : double(0);	
}

/**********************************************************************************************/
inline void VDK_Cursor::GetDate( ushort inIndex, 			
								 long& outYear, ushort& outMonth, ushort& outDay ) const
{ 
	VDK_Field* pField = GetField( inIndex );
	
	switch( pField->GetType() )
	{
		case kFBL_TypeDate:{
			VDK_Date* pDate = dynamic_cast<VDK_Date*> (pField);
			pDate->Get( outYear, outMonth, outDay );
		}break;
	
		case kFBL_TypeDateTime:{
			VDK_DateTime* pDateTime = dynamic_cast<VDK_DateTime*> (pField);
			pDateTime->GetDate( outYear, outMonth, outDay );
		}break;	
	}
}

/**********************************************************************************************/
inline void VDK_Cursor::GetDate( const char* inName, 
								 long& outYear, ushort& outMonth, ushort& outDay ) const
{
	VDK_Field* pField = GetField( inName );
	
	switch( pField->GetType() )
	{
		case kFBL_TypeDate:{
			VDK_Date* pDate = dynamic_cast<VDK_Date*> (pField);
			pDate->Get( outYear, outMonth, outDay );
		}break;
	
		case kFBL_TypeDateTime:{
			VDK_DateTime* pDateTime = dynamic_cast<VDK_DateTime*> (pField);
			pDateTime->GetDate( outYear, outMonth, outDay );
		}break;	
	}
}

/**********************************************************************************************/
inline void VDK_Cursor::GetTime( ushort inIndex, 			
								 ushort& outHour, ushort& outMinute, ushort& outSecond ) const
{ 
	VDK_Field* pField = GetField( inIndex );
	
	switch( pField->GetType() )
	{
		case kFBL_TypeTime:{
			VDK_Time* pDate = dynamic_cast<VDK_Time*> (pField);
			pDate->Get( outHour, outMinute, outSecond );
		}break;
	
		case kFBL_TypeDateTime:{
			VDK_DateTime* pDateTime = dynamic_cast<VDK_DateTime*> (pField);
			pDateTime->GetTime( outHour, outMinute, outSecond );
		}break;	
	}
}

/**********************************************************************************************/
inline void VDK_Cursor::GetTime( const char* inName, 
								 ushort& outHour, ushort& outMinute, ushort& outSecond ) const
{
	VDK_Field* pField = GetField( inName );
	
	switch( pField->GetType() )
	{
		case kFBL_TypeTime:{
			VDK_Time* pDate = dynamic_cast<VDK_Time*> (pField);
			pDate->Get( outHour, outMinute, outSecond );
		}break;
	
		case kFBL_TypeDateTime:{
			VDK_DateTime* pDateTime = dynamic_cast<VDK_DateTime*> (pField);
			pDateTime->GetTime( outHour, outMinute, outSecond );
		}break;	
	}
}


/**********************************************************************************************/
inline FBL_String* VDK_Cursor::GetString( ushort inIndex ) const{
	VDK_Field* pField = GetField( inIndex );
	return pField->GetString();	
} 

/**********************************************************************************************/
inline FBL_String* VDK_Cursor::GetString( const char* inName ) const{
	VDK_Field* pField = GetField( inName );
	return pField->GetString();	
}

/**********************************************************************************************/
inline PicHandle VDK_Cursor::GetPicture( ushort inIndex ) const{ 
	VDK_Picture* pPicture = GetPictureField( inIndex );
	return pPicture ? pPicture->ReadFromJPG() : NULL;		
}

/**********************************************************************************************/
inline PicHandle VDK_Cursor::GetPicture( const char* inName ) const{
	VDK_Picture* pPicture = GetPictureField( inName );
	return pPicture ? pPicture->ReadFromJPG() : NULL;		
}

#pragma mark -


/**********************************************************************************************/
inline VDK_Boolean* VDK_Cursor::GetBooleanField( ushort inIndex ) const{
	return dynamic_cast<VDK_Boolean*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Boolean* VDK_Cursor::GetBooleanField( const char* inName ) const{
	return dynamic_cast<VDK_Boolean*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Byte* VDK_Cursor::GetByteField( ushort inIndex ) const{
	return dynamic_cast<VDK_Byte*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Byte* VDK_Cursor::GetByteField( const char* inName ) const{
	return dynamic_cast<VDK_Byte*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Short* VDK_Cursor::GetShortField( ushort inIndex ) const{
	return dynamic_cast<VDK_Short*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Short* VDK_Cursor::GetShortField( const char* inName ) const{
	return dynamic_cast<VDK_Short*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_UShort* VDK_Cursor::GetUShortField( ushort inIndex ) const{
	return dynamic_cast<VDK_UShort*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_UShort* VDK_Cursor::GetUShortField( const char* inName ) const{
	return dynamic_cast<VDK_UShort*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Medium* VDK_Cursor::GetMediumField( ushort inIndex ) const{
	return dynamic_cast<VDK_Medium*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Medium* VDK_Cursor::GetMediumField( const char* inName ) const{
	return dynamic_cast<VDK_Medium*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_UMedium* VDK_Cursor::GetUMediumField( ushort inIndex ) const{
	return dynamic_cast<VDK_UMedium*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_UMedium* VDK_Cursor::GetUMediumField( const char* inName ) const{
	return dynamic_cast<VDK_UMedium*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Long* VDK_Cursor::GetLongField( ushort inIndex ) const{
	return dynamic_cast<VDK_Long*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Long* VDK_Cursor::GetLongField( const char* inName ) const{
	return dynamic_cast<VDK_Long*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_ULong* VDK_Cursor::GetULongField( ushort inIndex ) const{
	return dynamic_cast<VDK_ULong*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_ULong* VDK_Cursor::GetULongField( const char* inName ) const{
	return dynamic_cast<VDK_ULong*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_LLong* VDK_Cursor::GetLLongField( ushort inIndex ) const{
	return dynamic_cast<VDK_LLong*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_LLong* VDK_Cursor::GetLLongField( const char* inName ) const{
	return dynamic_cast<VDK_LLong*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_ULLong* VDK_Cursor::GetULLongField( ushort inIndex ) const{
	return dynamic_cast<VDK_ULLong*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_ULLong* VDK_Cursor::GetULLongField( const char* inName ) const{
	return dynamic_cast<VDK_ULLong*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Float* VDK_Cursor::GetFloatField( ushort inIndex ) const{
	return dynamic_cast<VDK_Float*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Float* VDK_Cursor::GetFloatField( const char* inName ) const{
	return dynamic_cast<VDK_Float*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Double* VDK_Cursor::GetDoubleField( ushort inIndex ) const{
	return dynamic_cast<VDK_Double*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Double* VDK_Cursor::GetDoubleField( const char* inName ) const{
	return dynamic_cast<VDK_Double*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Date* VDK_Cursor::GetDateField( ushort inIndex ) const{
	return dynamic_cast<VDK_Date*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Date* VDK_Cursor::GetDateField( const char* inName ) const{
	return dynamic_cast<VDK_Date*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Time* VDK_Cursor::GetTimeField( ushort inIndex ) const{
	return dynamic_cast<VDK_Time*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Time* VDK_Cursor::GetTimeField( const char* inName ) const{
	return dynamic_cast<VDK_Time*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_DateTime* VDK_Cursor::GetDateTimeField( ushort inIndex ) const{
	return dynamic_cast<VDK_DateTime*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_DateTime* VDK_Cursor::GetDateTimeField( const char* inName ) const{
	return dynamic_cast<VDK_DateTime*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_String* VDK_Cursor::GetStringField( ushort inIndex ) const{
	return dynamic_cast<VDK_String*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_String* VDK_Cursor::GetStringField( const char* inName ) const{
	return dynamic_cast<VDK_String*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_VarChar* VDK_Cursor::GetVarCharField( ushort inIndex ) const{
	return dynamic_cast<VDK_VarChar*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_VarChar* VDK_Cursor::GetVarCharField( const char* inName ) const{
	return dynamic_cast<VDK_VarChar*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_FixedBinary* VDK_Cursor::GetFixedBinaryField( ushort inIndex ) const{
	return dynamic_cast<VDK_FixedBinary*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_FixedBinary* VDK_Cursor::GetFixedBinaryField( const char* inName ) const{
	return dynamic_cast<VDK_FixedBinary*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_VarBinary* VDK_Cursor::GetVarBinaryField( ushort inIndex ) const{
	return dynamic_cast<VDK_VarBinary*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_VarBinary* VDK_Cursor::GetVarBinaryField( const char* inName ) const{
	return dynamic_cast<VDK_VarBinary*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_BLOB* VDK_Cursor::GetBLOBField( ushort inIndex ) const{
	return dynamic_cast<VDK_BLOB*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_BLOB* VDK_Cursor::GetBLOBField( const char* inName ) const{
	return dynamic_cast<VDK_BLOB*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Text* VDK_Cursor::GetTextField( ushort inIndex ) const{
	return dynamic_cast<VDK_Text*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Text* VDK_Cursor::GetTextField( const char* inName ) const{
	return dynamic_cast<VDK_Text*>( GetField(inName) );
}

/**********************************************************************************************/
inline VDK_Picture* VDK_Cursor::GetPictureField( ushort inIndex ) const{
	return dynamic_cast<VDK_Picture*>( GetField(inIndex) );
}

/**********************************************************************************************/
inline VDK_Picture* VDK_Cursor::GetPictureField( const char* inName ) const{
	return dynamic_cast<VDK_Picture*>( GetField(inName) );
}


#endif



