/**********************************************************************************************/
/* VDK_VarBinary.h																			  */
/*                                                                       					  */
/* Valentina SDK    																		  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/
/*
	Class VarBinary stores raw data of variable length from ZERO to MaxLength.
	Can be used as base class for a field that store Unicode strings.
	
	You can get/set value by copy operation GetValue/SetValue.
	Or you can use direct access to value buffer using GetValuePtr().
	
	Note, that FixedBinary/VarBinary (as well as String/VarChar) classes has own RAM buffer 
	to keep value. This is why for very big values may be better to use BLOB to 
	avoid double copying in RAM. 
*/

#ifndef __VDK_VarBinary_h 
	#define __VDK_VarBinary_h
	
#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/VDK_Field.h>
#else
#	include <VDK_Field.h>
#endif // VSDK_FRAMEWORK_STYLE

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
class FBL_EXP VDK_VarBinary : public virtual VDK_Field
{
		friend class VDK_BaseObject;

	public://///////////////////////////////////////////////////////////////////////////////////
		
FBL_EXP							VDK_VarBinary( 
									const char* inName, 
									ushort		inMaxLen	= 504, 
									ushort		inFlags		= 0 );

		/* Value methods: */

FBL_EXP	virtual ushort			GetLength( void ) const;

								// Read raw data from field into outVal buffer of size inBytes.
								// Returns number of bytes actually readed.
FBL_EXP virtual ushort          GetValue( void* outVal, ushort inBytes );

								// Write into field inBytes from inVal buffer.
FBL_EXP virtual void            SetValue( const void* inVal,  ushort inBytes );
						
						
		/* Properties methods: */						
									
FBL_EXP	virtual ushort			GetMaxLen( void ) const;			

								// Set the new length for this field.
								// This function modify size of record and Table must be rebuild.
FBL_EXP virtual void			SetMaxLen( ushort inMaxLen );


	protected://////////////////////////////////////////////////////////////////////////////////

FBL_EXP virtual void			FinishCreateSelf( void );


	protected://////////////////////////////////////////////////////////////////////////////////
		 
		ushort					mMaxLen;	
}; 


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif  


